#include <Python.h>
#include <stdio.h>
#include "nistp224.h"


static PyObject *nistp224_nistp224(PyObject *self, PyObject *args)
{
  unsigned char *x, *e;
  int len_x, len_e;
  unsigned char xe[28];
  int ret;

  if (!PyArg_ParseTuple(args, "s#s#", &x, &len_x, &e, &len_e))
    return NULL;

  if (len_x != len_e || len_x != 28) {
    PyErr_SetString(PyExc_ValueError, "passed strings must be 28 bytes long.");
    return NULL;
  }

  ret = nistp224(xe, x, e);
  if (ret == 0)
    return Py_BuildValue("s#", xe, 0);
  else
    return Py_BuildValue("s#", xe, 28);
}


static PyObject *nistp224_nistp224_56(PyObject *self, PyObject *args)
{
  unsigned char *xy, *e;	/* xy is 56 bytes, e is 28 */
  int len_xy, len_e;
  unsigned char xye[56];
  int ret;

  if (!PyArg_ParseTuple(args, "s#s#", &xy, &len_xy, &e, &len_e))
    return NULL;

  if (len_xy != 56) {
    PyErr_SetString(PyExc_ValueError, "xy must be 56 bytes long.");
    return NULL;
  }

  if (len_e != 28) {
    PyErr_SetString(PyExc_ValueError, "e must be 28 bytes long.");
    return NULL;
  }

  ret = nistp224_56(xye, xy, e);
  if (ret == 0)
    return Py_BuildValue("s#", xye, 0);
  else
    return Py_BuildValue("s#", xye, 56);
}


static PyObject *nistp224_nistp224_valid(PyObject *self, PyObject *args)
{
  unsigned char *in;		/* in is 56 bytes */
  int len_in;
  int ret;

  if (!PyArg_ParseTuple(args, "s#", &in, &len_in))
    return NULL;

  if (len_in != 56) {
    PyErr_SetString(PyExc_ValueError, "input must be 56 bytes long.");
    return NULL;
  }

  ret = nistp224_valid(in);
  return Py_BuildValue("i", ret);
}


static PyObject *nistp224_nistp224_uncompress(PyObject *self, PyObject *args)
{
  unsigned char *in;		/* 56 bytes */
  unsigned char inout[56];
  int len_in;
  int tmp, ret;

  /*
   * The C version works like this:
   * - do the work
   * - if the resultant value is invalid, set inout to 0s and return 0
   * - otherwise return 1. the calling function then has access to the
   *   uncompressed value.
   *
   * How we do it:
   * - take the value
   * - copy it into a variable named inout, so we don't mess with
   *   python's internal version of the input string
   * - call nistp224_uncompress() on it
   * - return whatever's in inout.
   */

  if (!PyArg_ParseTuple(args, "s#", &in, &len_in))
    return NULL;

  if (len_in != 56) {
    PyErr_SetString(PyExc_ValueError, "input must be 56 bytes long.");
    return NULL;
  }

  for (tmp = 0; tmp < 56; tmp++)
    inout[tmp] = in[tmp];

  ret = nistp224_uncompress(inout);

  return Py_BuildValue("s#", inout, 56);
}


static PyObject *nistp224_standard_base(PyObject *self, PyObject *args)
{
  return Py_BuildValue("s#", "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n", 28);
}

static PyObject *nistp224_standard56_base(PyObject *self, PyObject *args)
{

  return Py_BuildValue("s#", "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n1\r\0x1f\0xaf\0x11\0xbe\0xd9\0xda\0xde\0xd4\0xfe\0xeb\0x14s\0x9d\0xb8enr\0xd0\0xdc\0xceb0,\0xf1\0x8e\0x93", 56);
}


static PyMethodDef Nistp224Methods[] = {
  {"nistp224", nistp224_nistp224, METH_VARARGS,
   "Perform compressed point multiplication with a 28 byte compressed point and a 28 byte integer.


Pass the compressed point and integer as strings.

It can also be used for compressed D-H -- use nistp224.standard_base() as the
compressed point and a random 28 byte secret key."},
  {"nistp224_56", nistp224_nistp224_56, METH_VARARGS,
   "Perform uncompressed point multiplication with a 56 byte point and a 28 byte integer.

Pass the uncompressed point and integer as strings.

It can also be used for uncompressed D-H -- use nistp224.standard56_base() as the uncompressed point and a random 28 byte secret key."},
  {"nistp224_valid", nistp224_nistp224_valid, METH_VARARGS,
   "Check if a nistp224 point is valid.

Pass the point as a string of length 56."},
  {"nistp224_uncompress", nistp224_nistp224_uncompress, METH_VARARGS,
   "Uncompress a nistp224 point.

Pass the compressed point as a string of length 56.
Returns a zero'd string if uncompression was unsuccessful."},
  {NULL, NULL, 0, NULL}
};

void initnistp224(void)
{
  PyObject *standard_base_name;
  PyObject *standard_base;
  PyObject *standard56_base_name;
  PyObject *standard56_base;
  PyObject *d;
  PyObject *m;
  unsigned char st[28] = "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n";
  unsigned char st56[56] = "\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n1\r\x1f\xaf\x11\xbe\xd9\xda\xde\xd4\xfe\xeb\x14s\x9d\xb8" "enr\xd0\xdc\xce" "b0,\xf1\x8e\x93";

  m = Py_InitModule("nistp224", Nistp224Methods);
  d = PyModule_GetDict(m);

  standard_base_name = PyString_FromString("standard_base");
  standard56_base_name = PyString_FromString("standard56_base");

  standard_base = PyString_FromStringAndSize(st, 28);
  standard56_base = PyString_FromStringAndSize(st56, 56);

  PyDict_SetItem(d, standard_base_name, standard_base);
  PyDict_SetItem(d, standard56_base_name, standard56_base);
}

int main(int argc, char **argv)
{
  Py_SetProgramName(argv[0]);
  Py_Initialize();
  initnistp224();
  return 0;
}
